var Promise = require('any-promise');
var fs_1 = require('fs');
var path_1 = require('path');
var rimraf = require('rimraf');
var mkdirp = require('mkdirp');
var utils_1 = require('./utils');
var pipe_1 = require('./pipe');
var valve_1 = require('./valve');
/*
 * Register a new task.
 */
function task(name, pipes, directory, args) {
    if (directory === void 0) { directory = process.cwd(); }
    if (args === void 0) { args = utils_1.getArgs(); }
    var npipes = pipes.map(function (pipe) {
        if (pipe_1.isPlumber(pipe)) {
            return pipe.create();
        }
        return pipe;
    });
    var t = {
        name: name,
        pipes: npipes,
        directory: directory
    };
    return new Promise(function (resolve, reject) {
        var ran = false;
        if (t.name == (args._[0] || 'default')) {
            ran = true;
            runTask(t);
        }
        resolve(t);
    });
}
exports.task = task;
/*
 * Run a previously defined task.
 */
function runTask(t) {
    console.log("Running task " + t.name + "...");
    return new Promise(function (resolve) {
        var distPath = path_1.join(t.directory, 'dist');
        fs_1.stat(distPath, function (err, stats) {
            if (err == undefined) {
                rimraf(distPath, function (err) {
                    if (err) {
                        console.log('something error-y happened!');
                    }
                    fs_1.mkdirSync(distPath);
                    resolve();
                });
                return;
            }
            fs_1.mkdirSync(distPath);
            resolve();
        });
    })
        .then(function () {
        return utils_1.getFiles(t.directory);
    })
        .then(function (files) {
        var promises = files.map(function (filename) {
            return new Promise(function (resolve) {
                var v = new valve_1.Valve(fs_1.createReadStream(filename), filename, t.directory);
                for (var i = 0; i < t.pipes.length; i++) {
                    var pipe = t.pipes[i];
                    if (pipe.allows(filename)) {
                        v = pipe.do(v);
                    }
                }
                try {
                    var stats = fs_1.statSync(path_1.dirname(v.output));
                }
                catch (e) {
                    if (e.code == 'ENOENT') {
                        mkdirp.sync(path_1.dirname(v.output));
                    }
                }
                v.input.pipe(fs_1.createWriteStream(v.output));
                resolve();
            });
        });
        return Promise.all(promises)
            .then(function () {
            console.log("Finished running task " + t.name);
            return Promise.resolve();
        });
    });
}
exports.runTask = runTask;
